<?php
/*
 * (c) Logiless Inc.
 */

namespace Plugin\Logiless\Controller;

use Eccube\Application;
use Eccube\Entity\ProductClass;
use Eccube\Entity\ProductStock;
use Eccube\Repository\ProductClassRepository;
use Eccube\Repository\ProductStockRepository;
use Plugin\Logiless\Dao\ProductClassDao;
use Plugin\Logiless\E2V\ProductClassE2V;
use Plugin\Logiless\Exception\LogilessException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Annotation\Route;

/**
 * Class ProductClassesController
 * @package Plugin\Logiless\Controller
 */
class ProductClassesController extends AbstractController
{
    /**
     * @var ProductClassDao
     */
    protected $productClassDao;

    /**
     * @var ProductClassRepository
     */
    protected $productClassRepository;

    /**
     * @var ProductStockRepository
     */
    protected $productStockRepository;

    /**
     * OrdersController constructor.
     *
     * @param ProductClassDao $productClassDao
     * @param ProductClassRepository $productClassRepository
     * @param ProductStockRepository $productStockRepository
     */
    public function __construct(
        ProductClassDao $productClassDao,
        ProductClassRepository $productClassRepository,
        ProductStockRepository $productStockRepository
    ) {
        $this->productClassDao = $productClassDao;
        $this->productClassRepository = $productClassRepository;
        $this->productStockRepository = $productStockRepository;
    }

    /**
     * @Route("/plugin/logiless/product_classes.json", name="plugin_logiless_product_classes")
     * @param Request $request
     *
     * @return Response
     */
    public function index(Request $request)
    {
        return parent::index($request);
    }

    /**
     * @param Request $request
     *
     * @return JsonResponse
     * @throws LogilessException
     */
    protected function process(Request $request)
    {
        switch ($request->getMethod()) {
            case Request::METHOD_GET:
                return $this->onGet($request);

            case Request::METHOD_PATCH:
                return $this->onPatch($request);

            default:
        }
        throw new NotFoundHttpException();
    }

    /**
     * @param Request $request
     *
     * @return JsonResponse
     */
    protected function onGet(Request $request)
    {
        return $this->createResponse(ProductClassE2V::convert($this->productClassDao->getProductClasses($request)));
    }

    /**
     * @param Application $app
     * @param Request $request
     *
     * @return JsonResponse
     * @throws LogilessException
     */
    protected function onPatch(Request $request)
    {
        $data = json_decode($request->getContent(), true);

        if (is_null($data)) {
            throw LogilessException::onBadRequest(LogilessException::ERROR_INVALID_JSON_FORMAT);
        }

        if (!isset($data['product_classes'])) {
            throw LogilessException::onBadRequest(LogilessException::ERROR_INVALID_JSON_STRUCTURE);
        }

        foreach ($data['product_classes'] as $productClassData) {
            if (!isset($productClassData['code']) || !isset($productClassData['stock'])) {
                throw LogilessException::onBadRequest(LogilessException::ERROR_INVALID_JSON_STRUCTURE);
            }

            $productClasses = $this->productClassRepository->findBy(array('code' => $productClassData['code']));

            foreach ($productClasses as $productClass) {
                /* @var $productClass ProductClass */

                // 在庫情報の在庫数を更新
                $productStock = $this->productStockRepository->find($productClass->getProductStock()->getId());

                if ($productStock instanceof ProductStock) {
                    $productStock->setStock($productClassData['stock']);
                }

                // 商品規格情報の在庫数を更新
                $productClass->setStock($productClassData['stock']);
            }
        }

        $this->entityManager->flush();

        return $this->createResponse(array());
    }
}
