<?php
/*
 * (c) Logiless Inc.
 */

namespace Plugin\Logiless\Dao;

use Eccube\Entity\Order;
use Plugin\Logiless\Utils\StringUtils;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class OrderDao
 * @package Plugin\Logiless\Dao
 */
class OrderDao extends AbstractDao
{
    /**
     * 受注情報を取得
     *
     * @param Request $request
     *
     * @return Order[]
     */
    public function getOrders(Request $request)
    {
        $qb = $this->createQueryBuilder()
            ->select('o')
            ->from('Eccube\Entity\Order', 'o')
            ->orderBy('o.id', 'DESC');

        $criteria = static::buildCriteria($request, array(
            'id' => null,
            'ordered_at_from' => null,
            'ordered_at_to' => null,
            'status' => null,
        ));

        if (!is_null($criteria['id'])) {
            $idList = is_array($criteria['id']) ? $criteria['id'] : array($criteria['id']);

            $qb
                ->andWhere('o.id IN (:id_list)')
                ->setParameter('id_list', $idList);
        }

        if (!is_null($criteria['ordered_at_from'])) {
            $dateTime = StringUtils::convertStringToDateTime($criteria['ordered_at_from']);

            if ($dateTime instanceof \DateTime) {
                $qb
                    ->andWhere('o.order_date >= :ordered_at_from')
                   ->setParameter('ordered_at_from', $dateTime);
            }
        }

        if (!is_null($criteria['ordered_at_to'])) {
            $dateTime = StringUtils::convertStringToDateTime($criteria['ordered_at_to']);

            if ($dateTime instanceof \DateTime) {
                $qb
                    ->andWhere('o.order_date <= :ordered_at_to')
                    ->setParameter('ordered_at_to', $dateTime);
            }
        }

        if (!is_null($criteria['status'])) {
            $statusIdList = is_array($criteria['status']) ? $criteria['status'] : array($criteria['status']);

            $qb
                ->andWhere('o.OrderStatus IN (:status_id_list)')
                ->setParameter('status_id_list', $statusIdList);
        }

        $this->paging($qb, $request);

        return $qb->getQuery()->execute();
    }
}