<?php
/*
 * (c) Logiless Inc.
 */

namespace Plugin\Logiless\Exception;

use Exception;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class AbstractController
 * @package Plugin\Logiless\Controller
 */
class LogilessException extends Exception
{
    /**
     * @var string
     */
    const ERROR_AUTH_INVALID_TOKEN = 'A-0001';

    /**
     * @var string
     */
    const ERROR_AUTH_LOGIN_ID_IS_DUPLICATED = 'A-9001';

    /**
     * @var string
     */
    const ERROR_BAD_REQUEST = 'B-0001';

    /**
     * @var string
     */
    const ERROR_INVALID_JSON_FORMAT = 'B-0002';

    /**
     * @var string
     */
    const ERROR_INVALID_JSON_STRUCTURE = 'B-0003';

    /**
     * @var string
     */
    const ERROR_STATUS_ID_DOES_NOT_EXIST = 'B-0004';

    /**
     * @var string
     */
    protected $errorCode;

    /**
     * @var int
     */
    protected $httpStatus;

    /**
     * @param string $errorCode
     * @param int $httpStatus
     * @param \Exception $previous
     */
    public function __construct($errorCode, $httpStatus = Response::HTTP_INTERNAL_SERVER_ERROR, \Exception $previous = null)
    {
        $this->errorCode = $errorCode;
        $this->httpStatus = $httpStatus;

        parent::__construct($errorCode, 0, $previous);
    }

    /**
     * @param string $errorCode
     *
     * @return LogilessException
     */
    public static function onAccessDenied($errorCode)
    {
        return new self($errorCode, Response::HTTP_FORBIDDEN);
    }

    /**
     * @param string $errorCode
     *
     * @return LogilessException
     */
    public static function onBadRequest($errorCode)
    {
        return new self($errorCode, Response::HTTP_BAD_REQUEST);
    }

    /**
     * Get errorCode
     *
     * @return string
     */
    public function getErrorCode()
    {
        return $this->errorCode;
    }

    /**
     * Get httpStatus
     *
     * @return int
     */
    public function getHttpStatus()
    {
        return $this->httpStatus;
    }
}