<?php
/*
 * (c) Logiless Inc.
 */

namespace Plugin\Logiless\Utils;

/**
 * Class StringUtils
 * @package Plugin\Logiless\Utils
 */
class StringUtils
{
    /**
     * 日付文字列を DateTime オブジェクトに変換
     *
     * @param string $string 日付文字列
     * @param array $formats 日付フォーマット
     *
     * @return \DateTime|false
     */
    public static function convertStringToDate($string, array $formats = array())
    {
        if (empty($formats)) {
            $formats = array_merge(static::getDateFormats(), static::getDateTimeFormats());
        }

        foreach ($formats as $format) {
            $dateTime = \DateTime::createFromFormat($format, $string);

            if (
                $dateTime instanceof \DateTime
                && $dateTime->format($format) === $string && $dateTime->getTimestamp() > 0
            ) {
                $dateTime->setTime(0, 0, 0);

                return $dateTime;
            }
        }

        return false;
    }

    /**
     * 日時文字列を DateTime オブジェクトに変換
     *
     * @param string $string 日時文字列
     * @param array $formats 日時フォーマット
     *
     * @return \DateTime|false
     */
    public static function convertStringToDateTime($string, array $formats = array())
    {
        if (empty($formats)) {
            $formats = array_merge(static::getDateFormats(), static::getDateTimeFormats());
        }

        foreach ($formats as $format) {
            $dateTime = \DateTime::createFromFormat($format, $string);

            if (
                $dateTime instanceof \DateTime
                && $dateTime->format($format) === $string && $dateTime->getTimestamp() > 0
            ) {
                return $dateTime;
            }
        }

        return false;
    }

    /**
     * @return array
     */
    public static function getDateFormats()
    {
        return array(
            'Y-m-d',     // 2012-01-01
        );
    }

    /**
     * @return array
     */
    public static function getDateTimeFormats()
    {
        return array(
            'Y-m-d H:i:s',         // 2012-01-01 01:01:01
        );
    }
}